@@ -1,7 +1,10 @@
 (ns cemerick.pomegranate.aether-test
   (:require [cemerick.pomegranate.aether :as aether]
-            [clojure.java.io :as io])
-  (:use [clojure.test]))
+            [cemerick.pomegranate.test-report]
+            [clojure.java.io :as io]
+            [clojure.string :as str]
+            [clojure.test :refer [deftest is are testing use-fixtures]])
+  (:import (java.io File)))
 
 (deftest dependency-roundtripping
   (are [x] (= x (#'aether/dep-spec (#'aether/dependency x)))
@@ -20,19 +23,22 @@
 (def tmp-remote-repo {"tmp-remote-repo" (str "file://" tmp-remote-repo-dir)})
 
 (defn delete-recursive
-  [dir]
-  (when (.isDirectory dir)
-    (doseq [file (.listFiles dir)]
+  [^File file]
+  (when (.isDirectory file)
+    (doseq [file (.listFiles file)]
       (delete-recursive file)))
-  (.delete dir))
+  (when (not (.delete file))
+    (throw (ex-info (str "failed to delete: " (str file)) {}))))
 
 (defn- clear-tmp
   [f]
-  (delete-recursive (io/file tmp-dir)) (f))
+  (when (.exists ^File tmp-dir)
+    (delete-recursive tmp-dir))
+  (f))
 
 (use-fixtures :each clear-tmp)
 
-(defn file-path-eq [file1 file2]
+(defn file-path-eq [^File file1 ^File file2]
   (= (.getAbsolutePath file1)
      (.getAbsolutePath file2)))
 
@@ -51,12 +57,12 @@
                      [log4j "1.2.12"] nil,
                      [logkit "1.0.1"] nil}}]
     (is (= graph (aether/resolve-dependencies :coordinates deps :retrieve false :local-repo tmp-local-repo-dir)))
-    (is (not (some #(-> % .getName (.endsWith ".jar")) (file-seq tmp-local-repo-dir))))
+    (is (not (some #(-> ^File % .getName (.endsWith ".jar")) (file-seq tmp-local-repo-dir))))
 
     (doseq [[dep _] (aether/resolve-dependencies :coordinates deps :local-repo tmp-local-repo-dir)]
       (is (-> dep meta :file))
-      (is (-> dep meta :file .exists)))
-    (is (some #(-> % .getName (.endsWith ".jar")) (file-seq tmp-local-repo-dir)))
+      (is (-> dep meta ^File (:file) .exists)))
+    (is (some #(-> ^File % .getName (.endsWith ".jar")) (file-seq tmp-local-repo-dir)))
 
     (is (= hierarchy (aether/dependency-hierarchy deps graph)))))
 
@@ -69,13 +75,13 @@
                      :coordinates '[[demo "1.0.0"]] :retrieve false
                      :files {'[demo "1.0.0"] (io/file "test-repo" "demo" "demo" "1.0.0" "demo-1.0.0.jar")}
                      :local-repo tmp-local-repo-dir))))
-    (is (not (some #(-> % .getName (.endsWith ".jar"))
+    (is (not (some #(-> ^File % .getName (.endsWith ".jar"))
                    (file-seq tmp-local-repo-dir))))
     (doseq [dep (aether/resolve-artifacts
                      :coordinates deps :local-repo tmp-local-repo-dir)]
       (is (-> dep meta :file))
-      (is (-> dep meta :file .exists)))
-    (is (some #(-> % .getName (.endsWith ".jar"))
+      (is (-> dep meta ^File (:file) .exists)))
+    (is (some #(-> ^File % .getName (.endsWith ".jar"))
               (file-seq tmp-local-repo-dir)))))
 
 (deftest impl-detail-types
@@ -92,7 +98,7 @@
                                           :local-repo tmp-local-repo-dir)]
     (is (= 1 (count deps)))
     (is (= (.getAbsolutePath (io/file tmp-dir "local-repo" "javax" "servlet" "servlet-api" "2.5" "servlet-api-2.5.jar"))
-           (.getAbsolutePath (first (aether/dependency-files deps)))))))
+           (.getAbsolutePath ^File (first (aether/dependency-files deps)))))))
 
 (deftest resolve-deps-with-wildcard-mirror
   (let [deps (aether/resolve-dependencies :repositories {"clojars" "https://clojars.org/repo"}
@@ -101,7 +107,7 @@
                                           :local-repo tmp-local-repo-dir)]
     (is (= 1 (count deps)))
     (is (= (.getAbsolutePath (io/file tmp-dir "local-repo" "javax" "servlet" "servlet-api" "2.5" "servlet-api-2.5.jar"))
-           (.getAbsolutePath (first (aether/dependency-files deps)))))))
+           (.getAbsolutePath ^File (first (aether/dependency-files deps)))))))
 
 (deftest resolve-deps-with-wildcard-override-mirror
   (let [deps (aether/resolve-dependencies :repositories test-remote-repo
@@ -111,7 +117,7 @@
                                           :local-repo tmp-local-repo-dir)]
     (is (= 1 (count deps)))
     (is (= (.getAbsolutePath (io/file tmp-dir "local-repo" "javax" "servlet" "servlet-api" "2.5" "servlet-api-2.5.jar"))
-           (.getAbsolutePath (first (aether/dependency-files deps)))))))
+           (.getAbsolutePath ^File (first (aether/dependency-files deps)))))))
 
 (deftest resolve-deps
   (let [deps (aether/resolve-dependencies :repositories test-repo
@@ -119,7 +125,7 @@
                                           :local-repo tmp-local-repo-dir)]
     (is (= 1 (count deps)))
     (is (= (.getAbsolutePath (io/file tmp-dir "local-repo" "demo" "demo" "1.0.0" "demo-1.0.0.jar"))
-           (.getAbsolutePath (first (aether/dependency-files deps)))))))
+           (.getAbsolutePath ^File (first (aether/dependency-files deps)))))))
 
 (deftest resolve-deps-with-deps
   (let [deps (aether/resolve-dependencies :repositories test-repo
@@ -149,7 +155,7 @@
                                           :local-repo tmp-local-repo-dir)]
     (is (= 1 (count deps)))
     (is (= (.getAbsolutePath (io/file tmp-dir "local-repo" "demo" "demo2" "1.0.0" "demo2-1.0.0.jar"))
-           (.getAbsolutePath (first (aether/dependency-files deps)))))))
+           (.getAbsolutePath ^File (first (aether/dependency-files deps)))))))
 
 (deftest resolve-deps-with-classifiers
   (let [deps (aether/resolve-dependencies :repositories test-repo
@@ -158,10 +164,10 @@
                                           :local-repo tmp-local-repo-dir)]
     (is (= 1 (count deps)))
     (is (= (.getAbsolutePath (io/file tmp-dir "local-repo" "demo" "demo" "1.0.1" "demo-1.0.1-test.jar"))
-           (.getAbsolutePath (first (aether/dependency-files deps)))))))
+           (.getAbsolutePath ^File (first (aether/dependency-files deps)))))))
 
 (deftest resolve-managed-dependencies
-  (testing "supports coordinates w/o version number, with managed coordinates"
+  (testing "coordinate inherits version from managed coordinate"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo]]
@@ -171,7 +177,7 @@
       (is (= 1 (count files)))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo" "1.0.0" "demo-1.0.0.jar"))
                               files))))))
-  (testing "supports coordinates w/o version number, with managed coordinates, w/o group-id"
+  (testing "coordinate inherits version from managed coordinate using explicit/implicit group ids"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo]]
@@ -190,7 +196,7 @@
       (is (= 1 (count files)))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo" "1.0.0" "demo-1.0.0.jar"))
                               files))))))
-  (testing "supports coordinates w/nil version number, with managed coordinates"
+  (testing "coordinate with nil version inherits version from managed coordinate"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo nil]]
@@ -200,7 +206,7 @@
       (is (= 1 (count files)))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo" "1.0.0" "demo-1.0.0.jar"))
                               files))))))
-  (testing "supports coordinates w/nil version number and kwargs, with managed coordinates"
+  (testing "coordinate with nil version and kwargs inherits version from managed coordinate"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo2 nil :exclusions [demo/demo]]]
@@ -210,19 +216,19 @@
       (is (= 1 (count files)))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo2" "1.0.0" "demo2-1.0.0.jar"))
                               files))))))
-  (testing "error if missing version number w/o managed coordinates"
+  (testing "error if coordinate missing version number without a managed coordinate"
     (is (thrown-with-msg? IllegalArgumentException #"Provided artifact is missing a version: \[demo/demo\]"
                           (aether/resolve-dependencies
                            :repositories test-repo
                            :coordinates '[[demo/demo]]
                            :local-repo tmp-local-repo-dir))))
-  (testing "error if nil version number w/o managed coordinates"
+  (testing "error if coordinate has nil version number without managed coordinates"
     (is (thrown-with-msg? IllegalArgumentException #"Provided artifact is missing a version: \[demo/demo nil\]"
                           (aether/resolve-dependencies
                            :repositories test-repo
                            :coordinates '[[demo/demo nil]]
                            :local-repo tmp-local-repo-dir))))
-  (testing "coordinates version number overrides managed coordinates version"
+  (testing "coordinate version number overrides managed coordinates version"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo "1.0.0"]]
@@ -232,7 +238,7 @@
       (is (= 1 (count files)))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo" "1.0.0" "demo-1.0.0.jar"))
                               files))))))
-  (testing "managed coordinates version is honored for transitive deps"
+  (testing "managed coordinates version is applied to transitive deps"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo2 "1.0.0"]]
@@ -244,6 +250,98 @@
                               files))))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo" "1.0.1" "demo-1.0.1.jar"))
                               files))))))
+  (testing "coordinate scope overrides managed coordinate scope"
+    (testing "coordinate has nil version"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo nil :scope "test"]]
+                  :managed-coordinates '[[demo/demo "1.0.0"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= deps '{[demo "1.0.0" :scope "test"] nil}))))
+    (testing "coordinate overrides version"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo "1.0.1" :scope "test"]]
+                  :managed-coordinates '[[demo/demo "1.0.0"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= deps '{[demo "1.0.1" :scope "test"] nil}))))
+    (testing "managed coordinate specifies scope"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo "1.0.1" :scope "test"]]
+                  :managed-coordinates '[[demo/demo "1.0.0" :scope "provided"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= deps '{[demo "1.0.1" :scope "test"] nil})))
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo "1.0.1" :scope "provided"]]
+                  :managed-coordinates '[[demo/demo "1.0.0" :scope "test"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= deps '{[demo "1.0.1" :scope "provided"] nil})))))
+  (testing "coordinate transitive dep scope overrides unspecified managed-coordinate scope"
+    (testing "test overrides unspecified"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo2 "1.0.0" :scope "test"]]
+                  :managed-coordinates '[[demo/demo "1.0.1"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= '{[demo/demo2 "1.0.0" :scope "test"] #{[demo "1.0.1" :scope "test"]}
+                 [demo "1.0.1" :scope "test"] nil}
+               deps)))
+      (testing "compile overrides unspecified"
+        (let [deps (aether/resolve-dependencies
+                    :repositories test-repo
+                    :coordinates '[[demo/demo2 "1.0.0" :scope "compile"]]
+                    :managed-coordinates '[[demo/demo "1.0.1"]]
+                    :local-repo tmp-local-repo-dir)]
+          (is (= '{[demo/demo2 "1.0.0"] #{[demo "1.0.1"]}
+                   [demo "1.0.1"] nil}
+                 deps))))
+      (testing "implicit compile is effectively compile"
+        (let [deps (aether/resolve-dependencies
+                    :repositories test-repo
+                    :coordinates '[[demo/demo2 "1.0.0"]]
+                    :managed-coordinates '[[demo/demo "1.0.1"]]
+                    :local-repo tmp-local-repo-dir)]
+          (is (= '{[demo/demo2 "1.0.0"] #{[demo "1.0.1"]}
+                   [demo "1.0.1"] nil}
+                 deps)))))
+    (testing "compile does not override specified test"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo2 "1.0.0" :scope "compile"]]
+                  :managed-coordinates '[[demo/demo "1.0.1" :scope "test"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= '{[demo/demo2 "1.0.0"] #{[demo "1.0.1" :scope "test"]}
+                 [demo "1.0.1" :scope "test"] nil}
+               deps))))
+    (testing "unspecified (effectively compile) does not override specified test"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo2 "1.0.0"]]
+                  :managed-coordinates '[[demo/demo "1.0.1" :scope "test"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= '{[demo/demo2 "1.0.0"] #{[demo "1.0.1" :scope "test"]}
+                 [demo "1.0.1" :scope "test"] nil}
+               deps))))
+    (testing "provided does not override specified test"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo2 "1.0.0" :scope "provided"]]
+                  :managed-coordinates '[[demo/demo "1.0.1" :scope "test"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= '{[demo/demo2 "1.0.0" :scope "provided"] #{[demo "1.0.1" :scope "test"]}
+                 [demo "1.0.1" :scope "test"] nil}
+               deps))))
+    (testing "test does not override specified provided"
+      (let [deps (aether/resolve-dependencies
+                  :repositories test-repo
+                  :coordinates '[[demo/demo2 "1.0.0" :scope "test"]]
+                  :managed-coordinates '[[demo/demo "1.0.1" :scope "provided"]]
+                  :local-repo tmp-local-repo-dir)]
+        (is (= '{[demo/demo2 "1.0.0" :scope "test"] #{[demo "1.0.1" :scope "provided"]}
+                 [demo "1.0.1" :scope "provided"] nil}
+               deps)))))
   (testing "unused entries in managed coordinates are not resolved"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
@@ -255,7 +353,7 @@
       (is (= 1 (count files)))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo" "1.0.0" "demo-1.0.0.jar"))
                               files))))))
-  (testing "exclusions in managed coordinates are honored"
+  (testing "exclusions in managed coordinates are inherited"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo2]]
@@ -265,7 +363,7 @@
       (is (= 1 (count files)))
       (is (= 1 (count (filter #(file-path-eq % (io/file tmp-dir "local-repo" "demo" "demo2" "1.0.0" "demo2-1.0.0.jar"))
                               files))))))
-  (testing "classifiers in managed coordinates are honored"
+  (testing "classifiers in managed coordinates are inherited"
     (let [deps (aether/resolve-dependencies
                 :repositories test-repo
                 :coordinates '[[demo/demo]
@@ -299,6 +397,7 @@
   (let [repo-file (partial io/file "test-repo" "demo" "demo" "1.0.0")]
     (aether/deploy
      :coordinates '[group/artifact "1.0.0"]
+     ;;TODO was this first entry supposed be a jar file?
      :artifact-map {[] (repo-file "demo-1.0.0.pom")
                     [:extension "pom"] (repo-file "demo-1.0.0.pom")}
      :repository tmp-remote-repo
@@ -571,12 +670,12 @@
   <packaging>jar</packaging>
   <version>" version "</version>
   <name>" name "</name>"
-  (if-not (empty? deps)
+  (when-not (empty? deps)
     (apply str
            "<dependencies>"
-           (clojure.string/join "\n"
-                                (for [[n v] deps]
-                                  (str "<dependency>
+           (str/join "\n"
+                     (for [[n v] deps]
+                       (str "<dependency>
                    <groupId>" n "</groupId>
                    <artifactId>"n"</artifactId>
                    <version>"v"</version>
@@ -590,9 +689,9 @@
   <artifactId>" name "</artifactId>
   <versioning>
   <versions>"
-  (clojure.string/join "\n"
-                       (for [v versions]
-                         (str "<version>"v"</version>")))
+  (str/join "\n"
+            (for [v versions]
+              (str "<version>"v"</version>")))
     "</versions>
     <lastUpdated>20120810193549</lastUpdated>
   </versioning>
@@ -619,7 +718,7 @@
         (setTimeout [_ _])
         (setInteractive [_ _])
         (get [_ name file]
-          (let [[n _ version] (clojure.string/split name #"/")]
+          (let [[n _ version] (str/split name #"/")]
             (if (= name (str n "/" n "/maven-metadata.xml"))
               (if-let [versions (get-versions n fake-repo)]
                 (spit file (make-metadata n versions))
@@ -637,11 +736,11 @@
                                                  :checksum :ignore}}
                                  :local-repo tmp-local-repo-dir)
     (is (= #{"local-repo" "a" "1" "a-1.pom" "_remote.repositories" "a-1.jar"}
-           (set (map (memfn getName) (file-seq tmp-local-repo-dir)))))))
+           (set (map (memfn ^File getName) (file-seq tmp-local-repo-dir)))))))
 
 (comment
   "tests needed for:
   repository authentication
   repository policies
-  dependency options (scope/optional)
+  dependency options (optional)
   exclusion options (classifier/extension)")
